#include <stdlib.h>
#include <stdio.h>
#include <packet.h>
#include <plugin.h>
#include <decode.h>

struct linux_sll {
	unsigned short	sll_family;
	unsigned short	sll_protocol;
	int		sll_ifindex;
	unsigned short	sll_hatype;
	unsigned char	sll_pkttype;
	unsigned char	sll_halen;
	unsigned char	sll_addr[8];
};

/* SLL: Linux sockaddr link layer */
int sll_decode(struct packet *);
int sll_print(struct layer *, char *, int);
int lsll_decode(struct packet *);
int lsll_print(struct layer *, char *, int);
struct proto sll_p=init_proto_ex("sll", sll_decode, sll_print);
struct proto lsll_p=init_proto_ex("linux", lsll_decode, lsll_print);
struct proto_req lsll_r[]={
	null_request()
};
struct proto_req sll_r[]={
	proto_request("__pcap_dlt", 0x71),
	null_request()
};

char *ptype[]={
	"unicast",
	"broadcast",
	"multicast",
	"promisc",
	"outgoing",
};

int sll_name(unsigned short hatype, char *buf, int buflen)
{
	if ( !buf || !buflen ) return 0;

	if ( hatype==1)
		return snprintf(buf, buflen, "ethernet");
	if ( hatype==772)
		return snprintf(buf, buflen, "loopback");

	return snprintf(buf, sizeof(buf), "%u", hatype&0xffff);
}

int sll_print(struct layer *l, char *buf, int buflen)
{
	struct pkt_sllhdr *sll=l->h.sll;
	char *type;

	type=(sll->sll_pkttype<5) ?
		ptype[sll->sll_pkttype] :
		"unknown";

	return snprintf(buf, buflen,
		"%s",
		type);
}

int lsll_print(struct layer *l, char *buf, int buflen)
{
	struct linux_sll *sll=(struct linux_sll *)l->h.raw;
	char hdr[32];
	char *type;

	type=(sll->sll_pkttype<5) ?
		ptype[sll->sll_pkttype] :
		"unknown";

	sll_name(sll->sll_hatype, hdr, sizeof(hdr));

	return snprintf(buf, buflen,
		"if%i:%s - %s",
		sll->sll_ifindex,
		type,
		hdr);
}

int lsll_decode(struct packet *p)
{
	struct proto_child *pc;
	struct layer *l=&p->layer[p->llen-1];
	struct linux_sll *sll=(struct linux_sll *)l->h.raw;
	
	if ( p->llen >= PKT_LAYERS ) return 0;
	
	if ( (p->layer[p->llen].h.raw=
		l->h.raw+48)
		>= p->end ) return 0;

	p->layer[p->llen].flags=0;
	for(pc=l->proto->children; pc; pc=pc->next)
	{
		if ( sll->sll_protocol == pc->id ) {
			p->layer[p->llen++].proto=pc->proto;
			return pc->proto->decode(p);
		}
	}

	p->layer[p->llen++].proto=NULL;
	return 0;
}

int sll_decode(struct packet *p)
{
	struct proto_child *pc;
	struct layer *l=&p->layer[p->llen-1];
	
	if ( p->llen >= PKT_LAYERS ) return 0;
	
	if ( (p->layer[p->llen].h.raw=
		l->h.raw+sizeof(struct pkt_sllhdr))
		>= p->end ) return 0;

	p->layer[p->llen].flags=0;
	for(pc=l->proto->children; pc; pc=pc->next)
	{
		if ( l->h.sll->sll_protocol == pc->id ) {
			p->layer[p->llen++].proto=pc->proto;
			return pc->proto->decode(p);
		}
	}

	p->layer[p->llen++].proto=NULL;
	return 0;
}

int _PLUGIN_INIT (struct plugin_in *in, struct plugin_out *out)
{
	proc_decode_add decode_add;
	int ok=0;
	
	plugin_check(in, out);
	
	PLUGIN_ID("decode.sll", "Linux SLL");
	PLUGIN_VERSION(1, 0);
	PLUGIN_AUTHOR("Gianni Tedesco", "gianni@ecsc.co.uk");
	PLUGIN_LICENSE("GPL");
	
	if ( !(decode_add=in->import("decode.add")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	ok+=decode_add(&sll_p, sll_r);
	ok+=decode_add(&lsll_p, lsll_r);
	
	return (ok) ? PLUGIN_ERR_OK : PLUGIN_ERR_FAIL;
}

int _PLUGIN_UNLOAD (int code) {
	return PLUGIN_ERR_OK;
}
