/*
* This file is part of Firestorm NIDS
* Copyright (c) 2002 Gianni Tedesco
* This program is released under the terms of the GNU GPL version 2
*
*    This is the realm subsystem. It keeps track of all registered realms
*/
#include <rlim.h>
#include <stdio.h>
#include <string.h>
#include <packet.h>
#include <cleanup.h>
#include <signature.h>
#include <realm.h>

struct realm *realms=NULL;
unsigned int num_realms=0;
unsigned int cur_mask=1;

void realm_cleanup(int code, void *priv)
{
	struct realm *foo, *bar;
	
	for(foo=realms; foo;) {
		bar=foo;
		foo=foo->next;

		if ( bar->preproc ) free(bar->preproc);
		free(bar);
	}
}

void realm_init(void)
{
	cleanup_add(realm_cleanup, NULL);
}

/* Add a new realm */
struct realm *realm_add_r(
		int mask,
		const char *name, 
		proc_realm_match m,
		proc_realm_commit c)
{
	struct realm *ret;

	/* We can't have duplicates */
	if ( realm_find((char *)name) ) return NULL;

	if ( mask && (num_realms >= sizeof(cur_mask)*8) ) {
		printf("realm: error: Firestorm only supports "
			"%u realms\n", sizeof(cur_mask)*8);
		return NULL;
	}
	
	if ( !(ret=calloc(1, sizeof(struct realm))) ) {
		cperror("calloc");
	}

	/* Add it */
	ret->name=name;
	ret->next=realms;
	ret->mask=mask ? cur_mask : 0;
	ret->match=m;
	ret->commit=c;
	realms=ret;
	
	num_realms++;
	if ( mask ) cur_mask<<=1;
	
	return ret;
}

struct realm *realm_add(const char *name, proc_realm_match m, proc_realm_commit c)
{
	return realm_add_r(1, name, m, c);
}

struct realm *realm_add2(const char *name, proc_realm_match m, proc_realm_commit c)
{
	return realm_add_r(0, name, m, c);
}

/* Find a realm given it's name */
struct realm *realm_find(char *name)
{
	struct realm *r;

	for(r=realms; r; r=r->next) {
		if ( !strcmp(r->name, name) ) {
			return r;
		}
	}
	return NULL;
}
