/*
* This file is part of Firestorm NIDS
* Copyright (c) 2002 Gianni Tedesco
* This program is released under the terms of the GNU GPL version 2
*
*    This is the target subsystem. It manages alerts. Expect
*    this system to change dramatically...
*/
#include <rlim.h>
#include <stdio.h>
#include <string.h>
#include <packet.h>
#include <alert.h>
#include <target.h>

void b_target(struct generator *, struct packet *, struct alert*, void *);
int b_validate(char *, void **);
void b_cleanup(void *);
struct target target_builtin={
	next: NULL,
	name: "builtin",
	target: b_target,
	validate: b_validate,
	cleanup: b_cleanup,
};

struct target *targets=&target_builtin;
unsigned int num_targets=0;
unsigned int target_warn=0;

int b_validate(char *args, void **ptr) {return 1;}
void b_cleanup(void *ptr){}
void b_target(struct generator *gen,
	struct packet *p,
	struct alert *a,
	void *priv)
{
	printf("alert: %lu.%.6lu: %llu: %s: %s\n",
		p->time.tv_sec,
		p->time.tv_usec,
		p->serial,
		gen->name,
		a->alert);
}

/* Find a target by its name */
struct target *target_find(char *name)
{
	struct target *ret;

	for(ret=targets; ret; ret=ret->next)
	{
		if ( !strcmp(ret->name, name) )
			return ret;
	}
	return NULL;
}

/* Add a bunch of targets */
int target_add(struct target *t)
{
	int i;
	
	if ( !t ) return 0;

	if ( num_targets >= MAX_TARGETS ) {
		if ( !target_warn ) {
			target_warn++;
			printf("target: warning: MAX_TARGETS exceeded.\n");
		}
		return 0;
	}

	for(i=0; t[i].name; i++) {
		t[i].next=targets;
		targets=&t[i];
		num_targets++;
	}
	
	return i;
}
