#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <packet.h>
#include <capture.h>
#include <capdev.h>
#include <decode.h>
#include <alert.h>
#include <target.h>
#include <plugin.h>
#include <time.h>

int llen=16;

void a_alert(struct generator *, struct packet *, struct alert*, void *);
int alert_v(char *, void **);
void alert_c(void *);

struct target alert_t[]={
	target_init("ascii", a_alert, alert_v, alert_c),
	target_null()
};

void hex_dump(FILE *f, char *ptr, unsigned int len)
{
	int i, j;
	int line;
	unsigned char *tmp=(unsigned char *)ptr;

	for(j=0; j<len; ) {
		if ( j+llen > len ) {
			line=len-j;
		}else{
			line=llen;
		}

		fprintf(f, "%05X : ", j);

		for(i=0; i<llen; i++) {
			if ( i<line ) {
				if ( isprint(tmp[i]) ) {
					fprintf(f, "%c", tmp[i]);
				}else{
					fprintf(f, ".");
				}
			}else{
				fprintf(f, " ");
			}
		}
		
		for(i=0; i<line; i++) {
			fprintf(f, " %02X", tmp[i]);
		}

		j+=line;
		tmp+=line;
		fprintf(f, "\n");
	}
}

void a_alert(struct generator *gen,
	struct packet *pkt,
	struct alert *a,
	void *priv)
{
	int i;
	char ln[128];
	struct tm *rtime;
	FILE *f=(FILE *)priv;
	
	rtime=localtime(&pkt->time.tv_sec);
	strftime(ln, sizeof(ln)-1, "%Y-%m-%d %H:%M:%S", rtime);

	/* Basic alert information */
	fprintf(f, "   alert: %s.%.6lu [%s] %s\n",
		ln, pkt->time.tv_usec, gen->name, a->alert);

	/* Capture level stuff */
	fprintf(f, " capture: %s[%s]: len=%u caplen=%u\n",
		pkt->capture->capdev->name,
		pkt->capture->interface,
		pkt->len,
		pkt->caplen);
	
	/* Print all the layers */
	for(i=0; i<pkt->llen; i++) {
		if ( !pkt->layer[i].proto ) {
			fprintf(f, "    data: Application layer data (%u bytes)\n",
				pkt->end-pkt->layer[i].h.raw);
			continue;
		}

		if ( pkt->layer[i].proto->print ) {
			pkt->layer[i].proto->print(
				&pkt->layer[i], ln, 82);
		}else{
			sprintf(ln, "(no information)");
		}
		fprintf(f, "%8s: %s\n",
			pkt->layer[i].proto->name,
			ln);
	}

	/* hex dump the mofo */
	hex_dump(f, pkt->base, pkt->caplen);

	fprintf(f, "\n");
	fflush(f);
}

int alert_v(char *args, void **ptr)
{
	if ( !args ) {
		*ptr=stdout;
		return 1;
	}else{	
		FILE *f;

		if ( (f=fopen(args, "a")) ) {
			*ptr=f;
			return 1;
		}else{
			return 0;
		}
	}
}

void alert_c(void *priv)
{
	FILE *f=priv;

	if ( f != stdout ) fclose(f);
}

int _PLUGIN_INIT (struct plugin_in *in, struct plugin_out *out)
{
	proc_target_add target_add;
	int ok=0;
	
	plugin_check(in, out);
	
	PLUGIN_ID("target.ascii", "Print a message to stdout");
	PLUGIN_VERSION(1,0);
	PLUGIN_AUTHOR("Gianni Tedesco", "gianni@ecsc.co.uk");
	PLUGIN_LICENSE("GPL");
	
	if ( !(target_add=in->import("target.add")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	ok+=target_add(alert_t);
	
	return (ok) ? PLUGIN_ERR_OK : PLUGIN_ERR_FAIL;
}

int _PLUGIN_UNLOAD (int code) {
	return PLUGIN_ERR_OK;
}
